// Changes:
//	Modified Add(), and implemented Set to handle node setting semantics
//	not provided by the .NET library.
//
// Performance notes:
//	  This routine uses a simple insertion sort to handle calls to Add.
//	Each element is compared from right to left. If obj is smaller than the
//	current array object, that object is slid to the right.  Otherwise the hole
//	from the last slide operation is used to hold obj.
//	  Most of the calls to Add() will return -1, i.e. in the normal case only a
//	fraction of the items will be smaller than the largest item on the list.
//	This common case is recognized in the first comparison.  Iteration occurs
//	only for those items that belong on the list, so for the normal case this
//	operation is faster than it's strictly linear performance would suggest.


public class SortedLimitedList : ArrayList
{
	private SortedLimitedList ()
	{
	}

	int max;

	public SortedLimitedList (int maxElements)
		: base (maxElements)
	{
		max = maxElements;
	}

	//	Processes list from right to left, sliding each node that is greater
	//	than 'this' to the right.  The loop ends when either the first node is
	//	reached, meaning obj is a new minimum, or it's proper sorted position
	//	in the list is reached.
	//	Returns position of obj or -1 if obj was not placed.

	public override int Add (object obj)
	{
		int pos = Count;

		while (pos > 0 && ((IComparable)base[pos-1]).CompareTo (obj) >= 0) {
			if (pos < max) {
				Set (pos, base[pos-1]);
			}
			pos --;
		}

		if (pos < max) {
			Set (pos, obj);
		} else {
			pos = -1;
		}

		return pos;
	}

	// Sets the argument index to the argument object.
	// Replaces the node if it already exists,
	// adds a new node if at the end of the list,
	// does nothing otherwise.
	internal void Set (int idx, object obj)
	{
		if (idx < Count) {
			base[idx] = obj;
		} else if (idx == Count) {
			base.Add (obj);
		}
	}
}



